package org.astakhova.view;

import org.astakhova.data.*;
import org.astakhova.view.listener.*;
import org.astakhova.view.param.SColor;
import javax.swing.*;
import java.awt.*;
import javax.swing.event.*;
import java.awt.image.*;

/**
 * Makes a description of a diagram form
 * @author Astakhova Julia A.
 */
public class DiagramForm extends JInternalFrame {
	private IDiagram myDiagram;
	private static final long serialVersionUID = 1;
	private static final int xOffset = 30, yOffset = 30;
	private String myFile = null;
	private MainForm myOwner;
	private SetOfSelectedItems myUnderCursor = new SetOfSelectedItems();
	private Selection mySelectedBlock;
	private MouseInputAdapter myMouseL;
	private boolean myHasChanged = false;
	private CPBox myCPBox = null;
	
	/**
	 * Creates a new diagram form
	 * @param mf owner
	 * @param d diagram model
	 * @param title title of the form
	 * @param position number of the form
	 */
	DiagramForm(MainForm mf, IDiagram d, String title, int position) {
		super(title, true, true, true, true);
		myOwner = mf;
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		setSize(mf.getWidth()-xOffset*position-8, 
				mf.getHeight()-yOffset*position-96);
		setLocation(xOffset*position, yOffset*position);
		setLayout(null);
	
		myDiagram = (d == null) ? DiagramFactory.newDiagram() : d;
		drawDiagram();
		
		//Lets not to choose the button in the beginning of the session
		PointerMouseL pml = new PointerMouseL();
		pml.setForm(this);
		setMouseL(pml);
		
		addInternalFrameListener(new InternalFrameAdapter() {
			public void internalFrameClosing(InternalFrameEvent e) {
				setUnselected();
				DiagramForm.this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
				if (myHasChanged) {
					int res = JOptionPane.showConfirmDialog(DiagramForm.this, 
							"Save this diagram?");
					if (res == JOptionPane.YES_OPTION)
						(myOwner.new SaveAction()).actionPerformed(null);
					else if (res == JOptionPane.CANCEL_OPTION) {
						DiagramForm.this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
						return;
					}
				}
				
				myOwner.disableButtons();				
				super.internalFrameClosed(e);
			}
		});
		
		addKeyListener(new BlockLabelEditL(this));
	}
	
	private void drawDiagram() {
		//Drawing blocks
		for (INode n : myDiagram.getNodes()) {
			if (n instanceof IDataNode)
				getContentPane().add(new DataBlock((IDataNode)n));
			else if (n instanceof IConditionNode)
				getContentPane().add(new ConditionBlock((IConditionNode)n));
			else if (n instanceof IBeginNode)
				getContentPane().add(new BeginEndBlock((IBeginNode)n));
			else if (n instanceof IEndNode)
				getContentPane().add(new BeginEndBlock((IEndNode)n));
		}
			
		//Drawing links
		for (INode nn : myDiagram.getNodes()) {
			for (IArrow a : nn.getOutArrows())
				if ( a!= null )
					if (nn instanceof IConditionNode)
						getContentPane().add(
								new ConditionLink(a, 
									(nn.getOutArrows().indexOf(a) == 0)));
					else
						getContentPane().add(new Link(a));
		}
	}
	
	public IDiagram getDiagram() {
		return myDiagram;
	}

	/**
	 * Gets the name of the file to save the diagram
	 * @return the name of the file to save the diagram
	 */
	String getFile() {
		return myFile;
	}
	
	/**
	 * Sets the file to save the diagram to
	 * @return file name
	 */
	void setFile(String f) {
		myFile = f;
	}
	
	/**
	 * Free all selected items through the diagram
	 */
	public void setUnselected() {
		//Frees selected block
		if (mySelectedBlock != null) {
			JTextField tf = mySelectedBlock.getBlock().getEditableComponent();
			if (tf != null) {
				String s = tf.getText();
				String ss = mySelectedBlock.getBlock().getNode().getText();
				mySelectedBlock.getBlock().getNode().setText(s);
				getContentPane().remove(tf);
				mySelectedBlock.getBlock().setEditableComponent(null);
				if (!s.equals(ss))
					fixChanges();
			}
			mySelectedBlock.unSelect();
			getContentPane().remove(mySelectedBlock);
			mySelectedBlock = null;
			DiagramForm.this.repaint();
			return;
		}
		
		//if there are editable label on the arrow
		for (Component o : getContentPane().getComponents())
			if (o instanceof ConditionLink) {
				JTextField jtf = ((ConditionLink) o).getEditableComponent();
				if (jtf != null) {
					String s = jtf.getText();
					String ss = ((ConditionLink) o).getArrow().getLabel();
					((ConditionLink) o).getArrow().setLabel(s);
					getContentPane().remove(jtf);
					((ConditionLink) o).setEditableComponent(null);
					if (!s.equals(ss))
						fixChanges();
					DiagramForm.this.repaint();
				}
			}
	}
	
	/**
	 * Frees the diagram selection box 
	 */
	public void setUnBoxed() {
		if (myCPBox != null) {
			getContentPane().remove(myCPBox);
			myUnderCursor.clear();
			myCPBox = null;
			DiagramForm.this.repaint();
		}
	}
	
	/**
	 * Gets a current mouse listener
	 * @return a current mouse listener
	 */
	public MouseInputAdapter getMouseL() {
		return myMouseL;
	}
	
	/**
	 * Sets a mouse listener
	 * @param ma mouse listener for the diagram form
	 */
	public void setMouseL(MouseInputAdapter ma) {
		myMouseL = ma;
		addMouseMotionListener(ma);
		addMouseListener(ma);
	}
	
	/**
	 * Fixes that the diagram has changed
	 */
	public void fixChanges() {
		myHasChanged = true;
		if (title.indexOf('*') == -1)
			title += "*";
	}
	
	/**
	 * Fixes that diagram was saved
	 */
	public void fixSaveChanges() {
		myHasChanged = false;
		final int i = title.indexOf('*');
		if (i > - 1)
			setTitle(title.substring(0, i));
	}
	
	/**
	 * Checks if the diagram has not saved yet changes
	 * @return true if the diagram has not saved yet changes
	 */
	public boolean isChanged() {
		return myHasChanged;
	}
	
	/**
	 * Gets an image from the area diagram selection box includes
	 * @return an image from the area diagram selection box includes
	 */
	public BufferedImage getFrame() {
		if (myCPBox == null)
			return null;
		Rectangle r = myCPBox.getParameters();
		BufferedImage bi = new BufferedImage(this.getWidth(), this.getHeight(), 
				BufferedImage.TYPE_INT_RGB);
		Graphics g = bi.getGraphics();
		g.setColor(new Color(228, 235, 229));
		g.fillRect(0, 0, getWidth(), getHeight());
		
		for (Component o : myUnderCursor.getComponents())
			o.paint(g);
		bi = bi.getSubimage(r.x, r.y, r.width, r.height);
		g.dispose();
		
		return bi;
	}
	
	public Selection getSelectedBlock() {
		return mySelectedBlock;
	}
	
	public void setSelectedBlock(Selection s) {
		mySelectedBlock = s;
	}
	
	public SetOfSelectedItems getUnderCursor() {
		return myUnderCursor;
	}
	
	public void setUnderCursor(SetOfSelectedItems o) {
		myUnderCursor = o;
	}
	
	public CPBox getCPBox() {
		return myCPBox;
	}
	
	public void setCPBox(CPBox b) {
		myCPBox = b;
	}
	
	public MainForm getOwner() {
		return myOwner;
	}
	
	/**
	 * Selectes everything got into selection box frame
	 */
	public void selectCPBox() {
		Rectangle r = myCPBox.getParameters();
		myUnderCursor.clear();
		for (Component o : getContentPane().getComponents()) {
			if (o instanceof Block) {
				Block b = (Block) o;
				if (b.entersArea(r))
					myUnderCursor.addBlock(b);
				else
					myUnderCursor.removeBlock(b);
			}
		}
		for (Component o : getContentPane().getComponents()) {
			if (o instanceof AbstractLink) {
				AbstractLink l = (AbstractLink) o;
				INode s = l.getArrow().getSource();
				INode d = l.getArrow().getDest();
				boolean se = false, de = false;
				for (Component oo : getContentPane().getComponents()) {
					if (oo instanceof Block) {
						if (((Block) oo).getNode() == s && 
								((Block) oo).entersArea(r))
							se = true;
						if (((Block) oo).getNode() == d && 
								((Block) oo).entersArea(r))
							de = true;
					}
				}
				if (se && de)
					myUnderCursor.addLink(l);
				else
					myUnderCursor.removeLink(l);
			}
		}
		myUnderCursor.setSelected(SColor.selecting());
	}
}
